From d14f0d94b36176d5af13cd192901d544179f82f4 Mon Sep 17 00:00:00 2001
From: Toshikazu Nakayama <nakayama.ts@ncos.nec.co.jp>
Date: Tue, 18 Jan 2011 15:31:40 +0900
Subject: [PATCH] Add helper routine for load_module_symbols().

load_module_symbols_helper("module name", "module path in kernel tree") which
can wrap modref (mod_name), namelist (install path), base_addr (mod_base).

Signed-off-by: Toshikazu Nakayama <nakayama.ts@ncos.nec.co.jp>
---
 crash-5.1.1/defs.h    |    1 +
 crash-5.1.1/symbols.c |  107 +++++++++++++++++++++++++++++++++++++++++++++++++
 2 files changed, 108 insertions(+), 0 deletions(-)

diff --git a/crash-5.1.1/defs.h b/crash-5.1.1/defs.h
index 1b50fad..7873d4a 100755
--- a/crash-5.1.1/defs.h
+++ b/crash-5.1.1/defs.h
@@ -3648,6 +3648,7 @@ int get_function_numargs(ulong);
 int is_module_name(char *, ulong *, struct load_module **);
 int is_module_address(ulong, char *);
 int load_module_symbols(char *, char *, ulong);
+int load_module_symbols_helper(char *, char *);
 void delete_load_module(ulong);
 ulong gdb_load_module_callback(ulong, char *);
 char *load_module_filter(char *, int);
diff --git a/crash-5.1.1/symbols.c b/crash-5.1.1/symbols.c
index 2f7a6f6..5ff0c16 100755
--- a/crash-5.1.1/symbols.c
+++ b/crash-5.1.1/symbols.c
@@ -8928,6 +8928,113 @@ add_symbols:
 	return result;
 }
 
+int mod_path_exist(char *search_path)
+{
+	struct stat dmy;
+
+	if (stat(search_path, &dmy)) {
+		if (errno != ENOENT)
+			error(WARNING, "Can not stat(%s) errno=%d(!ENOENT).\n",
+			      search_path, errno);
+		/* file dose not exist. */
+		return 0;
+	}
+	return 1;
+}
+
+/*
+ * load_module_symbols() helper routine.
+ * Wrap the parts of modref, namelist, base_addr resolutions.
+ */
+int load_module_symbols_helper(char *mod_name, char *kernel_path)
+{
+	int i, mod_found = 0;
+	char *env, *obj_name, *check;
+	char search_path[PATH_MAX], version[65];
+	struct load_module *lm;
+
+	if ((env = getenv("CRASH_MODULE_PATH"))) {
+		/*
+		 * Try env at first.
+		 */
+		sprintf(search_path, "%s/%s/%s.ko", env, kernel_path, mod_name);
+		mod_found = mod_path_exist(search_path);
+		if (!mod_found) {
+			sprintf(search_path, "%s/%s/%s.o",
+				env, kernel_path, mod_name);
+			mod_found = mod_path_exist(search_path);
+		}
+	}
+
+	if (!mod_found) {
+		/*
+		 * Try /lib/modules/release-version/kernel path
+		 * Remove # SMP ... from utsname.version.
+		 */
+		sscanf(kt->utsname.version, "%s", version);
+		sprintf(search_path, "/lib/modules/%s-%s/kernel/%s/%s.ko",
+			kt->utsname.release, version, kernel_path, mod_name);
+		mod_found = mod_path_exist(search_path);
+		if (!mod_found) {
+			sprintf(search_path,
+				"/lib/modules/%s-%s/kernel/%s/%s.o",
+				kt->utsname.release, version,
+				kernel_path, mod_name);
+			mod_found = mod_path_exist(search_path);
+		}
+	}
+
+	if (!mod_found) {
+		/*
+		 * Try /lib/modules/release/kernel path
+		 */
+		sprintf(search_path, "/lib/modules/%s/kernel/%s/%s.ko",
+			kt->utsname.release, kernel_path, mod_name);
+		mod_found = mod_path_exist(search_path);
+		if (!mod_found) {
+			sprintf(search_path, "/lib/modules/%s/kernel/%s/%s.o",
+				kt->utsname.release, kernel_path, mod_name);
+			mod_found = mod_path_exist(search_path);
+		}
+	}
+
+	if (!mod_found)
+		/* give up. */
+		return FALSE;
+
+	obj_name = strdup(mod_name);
+	if (strstr(obj_name, "-")) {
+		check = obj_name;
+		while (*check) {
+			/*
+			 * module name is loaed from ELF section header
+			 * .gnu.linkonce.this_module.
+			 * module name string '-' is changed with "_".
+			 */
+			if (*check == '-')
+				*check = '_';
+			check++;
+		}
+	}
+
+	for (i = 0; i < st->mods_installed; i++) {
+		lm = &st->load_modules[i];
+		if (STREQ(obj_name, lm->mod_name)) {
+			free(obj_name);
+			if (!(lm->mod_flags & MOD_LOAD_SYMS))
+				return load_module_symbols(lm->mod_name,
+							   search_path,
+							   lm->mod_base);
+			else
+				return TRUE;
+		}
+	}
+
+	free(obj_name);
+	/* Not intalled. */
+	return FALSE;
+}
+
 /*
  *  Add a module's symbol file data to gdb's notion of the world.
  */
-- 
1.7.4.rc2

